

'Solar Analysis Tools -
 'Copyright (C) 2005 Gary Reysa  www.BuildItSolar.com  (gary@BuildItSolar.com)

 ' This program is free software; you can redistribute it and/or modify it
 ' under the terms of the GNU General Public License as published by the Free
 ' Software Foundation; either version 2 of the License, or (at your option)
 ' any later version.

 ' This program is distributed in the hope that it will be useful and encourage
 ' further development of solar tools, but   WITHOUT ANY WARRANTY; without even 
 ' the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 ' See the GNU General Public License   for more details.


'Example Solar Collector Simulation"  

'This is a simple example simulation that is a little more invoved than the
' "really simple example".  It makes use of the TMY, Sun and
'Collector classes to build the simulation. 

'The simulation models a collector bank, a heat storage tank using 
'water to store heat, a space to be heated, and a furnace as the
'alternate heat source.  It includes some primative report writting 
'routines to write the simulation results to a text file.

'While the simulation is quite simple, it still:
 '
'    - Use typical hour by hour weather for the city of interest 
'    - Calculates heat loss based on current weather
'    - Calculates collector output based on current sun intensity, sun 
'         direction, collector orientation, and temperatures of storage tank an ambient air
'    - Keeps track of temperature and heat stored in a storage tank
'    - Transfers heat from collector to storage and storage to house as needed
'    - Adds heat from a furnace if the sun is not providing enough

'A simulation along these lines could be used to estimate the 
'heat collected by a particualr brand of collector.
'It could be used to compare (say) the output of a evacuated tube collectors
'with a flat plate collector for your climate and storage configuration (you
'may be surprised at the outcome of this).

'It allows you to evalute tradeoffs between items such as buying more
'collectors, reducing heat loss, having more heat storage, using
'high tech or low tech collectors, and determine where you get the most bang-for-
'the-buck.

'It also provides the ability to programatically change the configuration
'of your simulation model on the fly during the simulation as a function of
'the current simulation conditions.
'For example, you could model a low thermal mass sunspace, and provide
'for transferring heat from the sunspace to the house when the sun is out, but 
'stopping the heat transfer when the sun is not out.  As Nick has pointed out,
'this is something Energy 10 is not capable of doing.  Or, model a window as 
'a collector with a thermal shade that is lowered at sunset and raised
'at sunrise.
'
' This simple example simulation models:
'    - 200 sqft of collectors using the efficiency curve
'      of a Heliodyne 410 collector with single glazing and
'      and a selective coating on the absorber.
'    - A thermal storage system with a thermal mass
'      equivalent to 4000 lb of water
'    - Weather for Billings, MT
'    - Building with a heat loss of 250 BTU/hr-F
'
'
'
'                                                     _
'      /-----------\                                 | |
'     /             \                             Collector (200 sqft)
'    /               \                               | |    
'    |   Space       |                               | |     
'    |   To          |                               | |     
'    |   Heat        |                               | |  Sun
'    |               |        +-----------+          | |<-----
'    |250 BTU/hr-F   |  Heat  | Heat      |          | |     
'    |loss rate      |<-------| Storage   |  Heat    | |     
'    |               |        | Tank(4Klb)|<---------| |     
'    |               |        +-----------+          |_|     
'-----------------------------------------------------------------
    ' The Main simulation loop works like this:
    '
    '     For each hour in simulation
    '         Get weather and sun intensity for this hour from TMY object
    '         Get sun direction for this hour from Sun object
    '         Get Collector heat output (if any) for this hour from Collector object
    '         Add collected heat to storage
    '         Calculate house heat loss for this hour
    '         Transfer heat to house from storage (if enough) and/or from furnace   
    '     Next Hour
    ' 
'
'  To keep the example small, the values of the inputs are assigned
'  in the source code.  To change them, just type in new values 
'  in the "Inputs:" section of the source, and hit the "run" key (F5?) to recompile and run
'  with the new inputs.  As long as you have Visual Studio up and running, this is 
'  faster than inputing numbers into a form anyway.
'
' !!NOTE!!:  The input weather file (Billings.TM2 for this example) must be in the current directory.
' And, the output file will be written to the current directory.
' Visual Basic seems to set the current directory to the \bin file for the project,
' so I put the weather file there, and, most likely, thats where the output will
' end up.  You might want to change these to full paths for your computer.

Imports System.io
Imports System.Math

Module OutHouse

' Record to hold hourly simulation results
' one of these is created for each hour as the simulation progresses
Public Structure OHstruc
    Public Code As Char
    Public Month As Integer
    Public Day As Integer
    Public Hour As Integer
    Public Tamb As Single
    Public Idn As Single
    Public Idif As Single
    Public Itot As Single
    Public Tstor As Single
    Public QColOut As Single
    Public Efic As Single
    Public HtFrac As Single
    Public QHouLoss As Single
    Public THou As Single
    Public Qfurnace As Single
End Structure

' This list of OHstruc's holds the full simulation results in memory
' The report writting routines sift through this list to find what
' they want and write it to the outut file.
Public OHlist As New ArrayList
Const DegPerRad As Single = 57.3

' variables used for inputs
Dim Title As String
Dim WxFileName As String
Dim OutFileName As String
Dim StartMonth As Integer
Dim EndMonth As Integer
Dim Longitude As Single
Dim Latitude As Single
Dim AltitudeMSL As Single
Dim HhouLoss As Single
Dim Thou As Single
Dim Whouse As Single
Dim ColArea As Single
Dim ColAzimuth As Single
Dim ColTilt As Single
Dim ReflecFactor As Single
Dim ColYIntercept As Single
Dim ColSlope As Single
Dim Wstor As Single


' Main simulation loop
Function Runsimulation() As Boolean

' INPUT VARIABLES -------------- MAKE CHANGES TO INPUTS HERE -------------------------'
     Title = "Example simulation -- Flat Plate Selective Collector"  ' Title of run
     OutFileName = "FPColSelAbsorbOut.txt"
     StartMonth = 10           ' Start simulation on first of this month
     EndMonth = 3              ' end simulation on last of this month
     WxFileName = "Billings.TM2"        ' Name of town for 
                  ' NOTE - this must be a file in the current directory, or a full path
     Longitude = 108.6 / DegPerRad   ' Longitude (rad)
     Latitude = 46 / DegPerRad 'Latitude (rad)
     AltitudeMSL = 5000        ' Altitude 
     Wstor = 4000              ' Thermal mass of storage system (lbs of water) 
     HhouLoss = 250            ' Heat loss in BTU/hr-F of heated space 
     Thou = 70                ' Temperature to keep house at
     ColArea = 200             ' Collector area in sqft
     ColAzimuth = 0 / DegPerRad   ' Azimuth of Collector (0 = south, + is to east  (rad)
     ColTilt = 90 / DegPerRad     ' Tilt angle of collector (rad) 
     ReflecFactor = 1.1        ' factor for snow reflectance
     ColYIntercept = 0.737     ' Yintercept on collector efficiency curve
     ColSlope = -0.805         ' Slope of collector efficiency curve

            ' Some sample collector data from the SRCC site and othe places:
            '     Use these to provide the ColYIntercept and ColSlope inputs
            '  Type          Glazing  absorber   Area      Brand          Y Intercept    Slope
            '  Flat Plate    Single   Painted    40        Heliodyne      0.726          -1.07
            '  Flat Plate    single   BlkChrm    40        Heliodyne      0.737          -0.805
            '  Flat Plate    double   selective  ?         MILHNBK        0.65           -0.66
            '  Flat Plate    single   selective  20        ACR Solar      0.604          -0.657  (polycarbonate glazing)
            '  evac tube     single   selective  14.5      Apricus Solar  0.418          -0.206   
            '  evac tube     single   selective  43        Sunda          0.529          -0.299
            '  evac tube     single   selective  46        Thermomax      0.574          -0.537
            '  Flat Plate    double   selective  40        CD program     0.63           -0.625
            '  Flat Plate    double   painted    40        CD program     0.66           -0.775
            '  Window        double   floor      40        ??              ??             ??

' END OF INPUTS ---------------------------------------------------------------

    Dim OH As OHstruc = New OHstruc()  ' holds current hour of input
    Dim TMY As TMYcls = New TMYcls()   ' to store hour by hour weather 
    Dim T As TMYstruc                 ' stores current hour of weather
    Dim STix, ETix As Integer
    Dim SV As New VectorCls          ' vector to store current direction to sun
    Dim Idn, Idif As Single          ' solar direct and diffuse radiation
    Dim HtFrac As Single            ' Solar heat fraction (sum of Gar and OH)
    Dim Efic As Single          ' collector efficiency to use
    Dim Cwater As Single = 1.0     ' specific heat of water (BTU/lb-F)
    Dim OHCol As New CollectorEcurveCls   ' collector with efficiency curve defined
    Dim Sun As New SunCls(1, StartMonth, 2001, 12, Longitude, Latitude, AltitudeMSL)
    Dim Tstor As Single  ' Storage tank temperature
    Dim TstorMin As Single ' Minimum allowed storage tank temperature
    Dim QStorAvail As Single  ' heat available in storage between Tstor and TstorMin
    Dim QHouLoss As Single ' house loss for the hour (BTU/hr)
    Dim QColOut As Single   ' Collector heat output for the hour (BTU/hr)
    Dim QFurnace As Single  ' Heat needed from furnace for current hour (BTU/hr)

    ' Set the values in ths Sun object that do not change during simulation
    Sun.Latitude = Latitude
    Sun.Longitude = Longitude
    Sun.Altitude = AltitudeMSL
    Sun.Year = 2001               ' just use 2001 
    ' Set the values in the Collector object that do not change during simulation
    OHCol.Area = ColArea  ' collector area
    OHCol.Azimuth = ColAzimuth   ' Collector azimuth (bearing)
    OHCol.Tilt = ColTilt         ' Collector tilt
    OHCol.Slope = ColSlope             ' Slope of the efficiency curve 
    OHCol.YIntercept = ColYIntercept   ' Y intercept of efficiency curves
    ' Set initial storage tank temperature
    Tstor = 100   ' start storage tank at 100F
    TstorMin = 90   ' mainimum storage tank temperature
                    ' no heat is transfered out of storage if tank is below this temperature

    'read in a TMY weather data set 
    If (Not TMY.ReadTMYFile(WxFileName)) Then
        Console.WriteLine("Could not find or read the TMY Weather file: " & WxFileName)
        Console.WriteLine("The weather file is probably not in the current directory")
        Console.WriteLine("Your Current Directory is: " & Directory.GetCurrentDirectory)
        Return False
    End If
    'Set up the start and end points for the sim in the weather data
    STix = TMY.TMYrecIndexForDate(StartMonth, 1, 1)
    ETix = TMY.TMYrecIndexForDate(EndMonth, 30, 1)  ' aprox the end of the last month
    Dim Tix As Integer = STix

    ' Main simulation loop:
    ' works like this:
    '     For each hour in simulation
    '         Get weather and sun intensity for this hour from TMY object
    '         Get sun direction for this hour from Sun object
    '         Get Collector heat output (if any) for this hour from Collector object
    '         Add collected heat to storage
    '         Calculate house heat loss for this hour
    '         Transfer heat to house from storage (if enough) and/or from furnace   
    '     Next Hour
    ' 

    While Tix <> ETix
                ' Get the weather for the hour 
                T = TMY.TMYrec(Tix)

                'SUN: 
                ' Update the sun
                Sun.DayOfMonth = T.Day
                Sun.Month = T.Month
                Sun.ASThr = T.Hour
                ' Get the direction to the sun, and solar intensity levels
                SV = Sun.ToSunVector()
                Idn = T.Idn
                Idif = T.Ihorzdif * ((1 + Cos(OHCol.Tilt))) / 2
                ' factor up sun for snow reflection
                Idn = Idn * ReflecFactor
                        ' not exactly the right way to do this, but better than not doing it

                ' HOUSE:
                ' calc heat loss for the house for this hour
                ' This amount of heat must be added from therma storage
                ' or by furnace to keep house temperature at Thou
                QHouLoss = HhouLoss * (Thou - T.Tdb)
                If QHouLoss < 0 Then QHouLoss = 0.1 ' hot day

                ' COLLECTOR:
                ' Update the Collector object to this hour
                OHCol.SunVector = SV      ' tell it sun direction
                OHCol.Idn = Idn           ' tell it sun intensity
                OHCol.Idif = Idif
                OHCol.Ta = T.Tdb          ' tell it outside temperature
                OHCol.Ti = Tstor          ' tell what its inlet temperature will be


                ' Get collector outputs
                Efic = OHCol.Efic
                QColOut = OHCol.QOut

                'STORAGE:
                ' Add any collected heat to storage 
                Tstor = Tstor + QColOut / Wstor

                ' Transfer heat to house from storage and/or furnace as necessary:
                '   Take heat from OH to get house up to ThouLowLimit 
                '   If this is still not enough, then turn furnace on to get house up to ThouLowLimit

                QStorAvail = (Tstor - TstorMin) * Wstor * Cwater
                If QStorAvail > QHouLoss Then
                    ' more than enough heat in storage -- transfer some to heat house
                    Tstor = Tstor - QHouLoss / (Wstor * Cwater)
                    QFurnace = 0
                Else
                    ' not enough heat in storage -- transfer whats available and add furnace
                    Tstor = TstorMin    ' discharges storage to min allowable temperature
                    QFurnace = QHouLoss - QStorAvail
                End If

                ' Calc solar heating fraction for the hour
                HtFrac = QColOut / QHouLoss
                    ' note that is will be > 1 for sunny hours -- excess goes to storage

            If Tix = 8760 Then
                Tix = 1        ' start new year if at end of year
            Else
                Tix += 1       'otherwise just move to next hour
            End If

            ' save the data for this hour
            OH.Code = "H"
            OH.Month = T.Month
            OH.Day = T.Day
            OH.Hour = T.Hour
            OH.Tamb = T.Tdb
            OH.Idn = Idn
            OH.Idif = Idif
            OH.Itot = Idn + Idif
            OH.Tstor = Tstor
            OH.QColOut = QColOut
            OH.Efic = Efic
            OH.HtFrac = HtFrac
            OH.QHouLoss = QHouLoss
            OH.THou = Thou
            OH.Qfurnace = QFurnace

           ' add this hour to OH list
            OHlist.Add(OH)
    End While
    Return True

End Function  ' One Day Sim



' Report writting routines:
' After the simulation loop has completed, all of the data from the simulation
' will be in memory in the OHlist arraylist.  
' These reporting routines scan the OHlist and pick out the things you want to
' send to the output file.
' Just call which ever of them you want to get the output you want.

' Writes the simulation inputs to the output file
Sub WriteOHDef(ByVal f As StreamWriter)
    f.WriteLine()
    f.WriteLine("Title: " & Title & " Town: " & WxFileName)
    f.WriteLine("House: Loss = {0,8:F0} ", HhouLoss)
    f.WriteLine("Collector: Area = {0,8:F0}  Azimuth = {1,6:F0}  Tilt = {2,6:F0} Yincept = {3,6:F2} Slope = {4,6:F2}", _
               ColArea, ColAzimuth * DegPerRad, ColTilt * DegPerRad, ColYIntercept, ColSlope)
    f.WriteLine("Storage: Wstor = {0,8:F0} ", Wstor)
    f.WriteLine()
End Sub

' Writes some selected summary information to the output file
Sub WriteSummaryStuff(ByVal f As StreamWriter)
    f.WriteLine()
    f.WriteLine("TstorHigh    = {0,6:F1}    TstorLow    = {1,6:F1}", TstorHigh(), TstorLow())
    f.WriteLine("TambientHigh = {0,6:F1}    TambientLow = {1,6:F1}", TambHigh(), TambLow())
    'Add more?
    f.WriteLine()
End Sub


' Writes the column titles for the hourly output sections in the output file
Sub WriteOHHeader(ByVal f As StreamWriter)
    f.WriteLine( _
"  Date----------  Temp    Sun-------  Storage  Collector ----------  House---------  Furnace")
    f.WriteLine( _
"  Mon   Day   Hr  Tamb    Idn   Idif  Tstor    QColOut   Efic  Frac  QhouLoss  Thou  Qfurnace")
End Sub

' Writes the column titles for the day, month and year summary lines
Sub WriteOHSummaryHeader(ByVal f As StreamWriter)
    f.WriteLine( _
"  Mon   Day                                    QColOut         Frac  QhouLoss        Qfurnace")
End Sub

' Writes out a line corresponding to a single line in the OHlist (usually a day, month or year summary)
Sub WriteOHSummary(ByVal OHrec As OHstruc, ByVal f As StreamWriter)
    f.WriteLine("{0,1:C1}" & _
    "{1,4:F0}  {2,4:F0}                        " & _
    "          {3,9:F0}        {4,5:F2} {5,9:F0}       " & _
    "{6,8:F0} ", _
    OHrec.Code, _
    OHrec.Month, OHrec.Day, _
                 OHrec.QColOut, OHrec.HtFrac, OHrec.QHouLoss, _
    OHrec.Qfurnace)
End Sub


' Writes out a line corresponding to a single line in the OHlist (usually one hours output)
Sub WriteOH(ByVal OHrec As OHstruc, ByVal f As StreamWriter)
    f.WriteLine("{0,1:C1}" & _
    "{1,4:F0}  {2,4:F0} {3,4:F0} {4,5:F1} {5,6:F1} {6,6:F1}" & _
    "{7,7:F1}  {8,9:F0}  {9,5:F2} {10,5:F2} {11,9:F0} {12,5:F0} " & _
    "{13,8:F0} ", _
    OHrec.Code, _
    OHrec.Month, OHrec.Day, OHrec.Hour, OHrec.Tamb, OHrec.Idn, OHrec.Idif, _
    OHrec.Tstor, OHrec.QColOut, OHrec.Efic, OHrec.HtFrac, OHrec.QHouLoss, OHrec.THou, _
    OHrec.Qfurnace)
End Sub

' Writes out every hourly record in the OHlist -- this will produce 
' about 700 lines of output per month -- good to scan through to see
' whats happening on a day by day basis as different types of weather
' roll by.
Sub YearByHour(ByVal DoSums As Boolean, ByVal f As StreamWriter)
    For Each I As OHstruc In OHlist
        WriteOH(I, f)
        If I.Hour = 24 Then
            If DoSums Then
                DaySum(I.Month, I.Day, f)
            End If
            WriteOHHeader(f)
        End If
    Next
End Sub

' Writes out every hourly record in OHlist for the passed month
Sub MonthByHour(ByVal Month As Integer, ByVal f As StreamWriter)
    For Each I As OHstruc In OHlist
        If I.Month = Month Then WriteOH(I, f)
    Next
End Sub

' Writes out every hourly record in the OHlist for the passed day
Sub DayByHour(ByVal Month As Integer, ByVal Day As Integer, ByVal f As StreamWriter)
    For Each I As OHstruc In OHlist
        If I.Month = Month And I.Day = Day Then WriteOH(I, f)
    Next
End Sub

' Writes out a one line summary for the day passed 
Sub DaySum(ByVal Month As Integer, ByVal Day As Integer, ByVal f As StreamWriter)
    Dim S As New OHstruc

    For Each I As OHstruc In OHlist
        If I.Month = Month And I.Day = Day Then
            S.QColOut += I.QColOut
            S.QHouLoss += I.QHouLoss
            S.Qfurnace += I.Qfurnace

        End If
    Next
    S.HtFrac = S.QColOut / S.QHouLoss
    S.Month = Month
    S.Day = Day
    S.Code = "D"
    WriteOHSummary(S, f)
End Sub

' Writes out a one line summary for the month passed
Sub MonthSum(ByVal Month As Integer, ByVal f As StreamWriter)
    Dim S As New OHstruc

    For Each I As OHstruc In OHlist
        If I.Month = Month Then
            S.QColOut += I.QColOut
            S.QHouLoss += I.QHouLoss
            S.Qfurnace += I.Qfurnace
        End If
    Next
    S.HtFrac = S.QColOut / S.QHouLoss

    S.Month = Month
    S.Code = "M"
    If S.QColOut <> 0 And S.QHouLoss <> 0 Then
        WriteOHSummary(S, f)
    End If
End Sub

' Writes out a one line summary for the full length of the simulation 
Sub YearSum(ByVal f As StreamWriter)
    Dim S As New OHstruc

    For Each I As OHstruc In OHlist
            S.QColOut += I.QColOut
            S.QHouLoss += I.QHouLoss
            S.Qfurnace += I.Qfurnace
    Next
    S.HtFrac = S.QColOut / S.QHouLoss
    S.Code = "Y"
    WriteOHSummary(S, f)
End Sub

' Scan OHlist to find the lowest ambient temperature
Function TambLow() As Single
    Dim Tlo As Single = +500
    For Each I As OHstruc In OHlist
        If I.Tamb < Tlo Then Tlo = I.Tamb
    Next
    TambLow = Tlo
End Function

Function TambHigh() As Single
    Dim Thi As Single = -100
    For Each I As OHstruc In OHlist
        If I.Tamb > Thi Then Thi = I.Tamb
    Next
    TambHigh = Thi
End Function

' Scan OHlist to find the highest value of direct normal sunlight
Function IdnHigh() As Single
    Dim IdnHi As Single = -50
    For Each I As OHstruc In OHlist
        If I.Idn > IdnHi Then IdnHi = I.Idn
    Next
    IdnHigh = IdnHi
End Function

' Scan the OHlist to find the highest storage temperature over the simulation
Function TstorHigh() As Single
    Dim TstorHi As Single = -50
    For Each I As OHstruc In OHlist
        If I.Tstor > TstorHi Then TstorHi = I.Tstor
    Next
    TstorHigh = TstorHi
End Function

Function TstorLow() As Single
    Dim TstorLo As Single = +500
    For Each I As OHstruc In OHlist
        If I.Tstor < TstorLo Then TstorLo = I.Tstor
    Next
    TstorLow = TstorLo
End Function

' Write selected simulation results to the output file
' You can change this to whatever floats your boat
Sub DoSimulationReport()
    ' open the output file:
    ' The file will be created on what every your current directory is
    Dim sw As New StreamWriter(OutFileName)
    Console.WriteLine("writing output to: " & Directory.GetCurrentDirectory & "\" & OutFileName)

    ' Write the simulation inputs to output
    WriteOHDef(sw)
    ' Write some calculated summary info (like lowest ambient) to output
    WriteSummaryStuff(sw)

    ' write the month by month summary
    sw.WriteLine()
    sw.WriteLine("Month Summaries:")
    WriteOHSummaryHeader(sw)
    For i As Integer = 1 To 12
        MonthSum(i, sw)
    Next
    ' Year sum at the bottom of the monthly summary

    sw.WriteLine()
    sw.WriteLine("Year Summary:")
    YearSum(sw)
    sw.WriteLine()

    'Do hour by hour reports for a couple days of interest

    sw.WriteLine()
    sw.WriteLine("Hourly output for a couple days that look interesting:")
    sw.WriteLine("Warmish fall day with good sun -- tank mid temp -")
    WriteOHHeader(sw)
    DayByHour(10, 9, sw)
    DaySum(10, 9, sw)
    sw.WriteLine("Very Cold, but clear December day")
    WriteOHHeader(sw)
    DayByHour(12, 10, sw)
    DaySum(12, 10, sw)

    ' Write the hour by hour results for the entire span of simulation
    sw.WriteLine()
    sw.WriteLine("Hourly Output for the entire simulation: ")
    WriteOHHeader(sw)
    YearByHour(True, sw)
    sw.Close()
End Sub

    Sub Main()
        Console.BufferWidth = 140
        Console.BufferHeight = 500

        If (Runsimulation()) Then
            DoSimulationReport()
        End If

        Console.WriteLine("Done")
        Console.WriteLine("Hit <enter> to continue")
        Console.ReadLine()
    End Sub

End Module

